/*
    pcfg.h
    Bayesian PCFG class.
    $Id: pcfg.h,v 1.18 2015/03/07 03:29:56 daichi Exp $

*/
#ifndef PCFG_PCFG_H
#define PCFG_PCFG_H
#include <cstdio>
#include <vector>
#include "dict.h"
#include "tree.h"
#include "grammar.h"

using namespace std;

struct PCFG {
	int K;
	Grammar *grammar;
	dictionary *dict;
	vector<Tree *> tree;
	vector<vector<vector<double> > > alpha;

	bool read (char *file, vector<vector<int> > &data);
	bool train (vector<vector<int> > &data, int iter);
	void compute_cky (vector<int> &s);
	double cky_sum (int x, int i, int n);

	Tree *parse (vector<int> &s);
	Tree *sample (vector<int> &s, int x, int i, int k);
	double treelik (Tree *tree);
	double polyalik (Tree *tree);

	bool add_counts (Tree *tree);
	bool remove_counts (Tree *tree);
	
	double expand (int x, int w);
	double expand (int x, int y, int z);

	bool save (char *model);
	bool save_dict (char *model);
	bool save_tree (char *model);
	bool save_grammar (char *model);

	bool write (char *model);
	bool write_grammar (char *model);
    
	/* constructor / destructor */
	PCFG (int K_, double gamma, double eta) {
		K = K_;
		dict = new dictionary;
		grammar = new Grammar (K, gamma, eta);
		fprintf(stderr, "PCFG initialized.\n");
	};
	~PCFG () {
		delete dict;
		delete grammar;
	};
};


#endif


