/*
    pcfgs.h
    Bayesian PCFG with slice sampler. (Makino & Takei 2009)
    $Id: pcfgs.h,v 1.8 2015/03/07 03:29:56 daichi Exp $

*/
#ifndef PCFG_PCFGS_H
#define PCFG_PCFGS_H
#include <cstdio>
#include <vector>
#include "dict.h"
#include "tree.h"
#include "grammar.h"

using namespace std;

struct PCFGS {
	int K;
	Grammar *grammar;
	dictionary *dict;
	vector<Tree *> tree;
	vector<vector<double> > slices;
	vector<vector<vector<double> > > alpha;

	bool read (char *file, vector<vector<int> > &data);
	bool train (vector<vector<int> > &data, int iter);

	Tree *parse (vector<int> &s, Tree *tree, vector<double> &slice);
	Tree *sample (vector<int> &s, int x, int i, int k, vector<double> &slice);
	void set_slice (Tree *tree, vector<double> &slice);
	void compute_cky (vector<int> &s, vector<double> &slice);
	double cky_sum (int x, int i, int n, vector<double> &slice);
	double treelik (Tree *tree);
	double polyalik (Tree *tree);

	bool add_counts (Tree *tree);
	bool remove_counts (Tree *tree);
	
	double expand (int x, int w);
	double expand (int x, int y, int z);

	bool save (char *model);
	bool save_dict (char *model);
	bool save_tree (char *model);
	bool save_grammar (char *model);

	/* constructor / destructor */
	PCFGS (int K_, double gamma, double eta) {
		K = K_;
		dict = new dictionary;
		grammar = new Grammar (K, gamma, eta);
		fprintf(stderr, "PCFGS initialized.\n");
	};
	~PCFGS () {
		delete dict;
		delete grammar;
	};
};


#endif


