/*
    pcfgt.h
    Bayesian PCFG with slice sampler. (Blunsom & Cohn 2010)
    $Id: pcfgt.h,v 1.8 2015/03/07 03:29:56 daichi Exp $

*/
#ifndef PCFG_PCFGT_H
#define PCFG_PCFGT_H
#include <cstdio>
#include <vector>
#include "dict.h"
#include "tree.h"
#include "grammar.h"

using namespace std;

struct PCFGT {
	int K;
	double alphab;	/* Beta prior of slice */
	double betab;
	Grammar *grammar;
	dictionary *dict;
	vector<Tree *> tree;
	vector<vector<double> > slice;
	vector<vector<vector<double> > > alpha;

	bool read (char *file, vector<vector<int> > &data);
	bool train (vector<vector<int> > &data, int iter);

	Tree *parse (vector<int> &s, Tree *tree);
	Tree *sample (vector<int> &s, int x, int i, int k);
	void init_slice (Tree *tree, int L);
	void set_slice (Tree *tree, int L);
	void compute_cky (vector<int> &s);
	double cky_sum (int x, int i, int n, double theta);
	double treelik (Tree *tree);
	double polyalik (Tree *tree);

	bool add_counts (Tree *tree);
	bool remove_counts (Tree *tree);
	
	double expand (int x, int w);
	double expand (int x, int y, int z);

	bool save (char *model);
	bool save_dict (char *model);
	bool save_tree (char *model);
	bool save_grammar (char *model);

	/* constructor / destructor */
	PCFGT (int K_, double gamma, double eta, double alpha_, double beta_) {
		K = K_;
		alphab = alpha_;
		betab  = beta_;
		dict = new dictionary;
		grammar = new Grammar (K, gamma, eta);
		fprintf(stderr, "PCFGT initialized.\n");		
	};
	~PCFGT () {
		delete dict;
		delete grammar;
	};
};


#endif


