/*
    tree.cpp
    PCFG Tree structure.
    $Id: tree.cpp,v 1.7 2014/12/17 11:22:24 daichi Exp $

*/
#include <string>
#include <sstream>
#include <iostream>
#include "tree.h"
#include "dict.h"

using namespace std;

int
Tree::nodes ()
{
	if (left && right)
		return left->nodes() + right->nodes();
	else
		return 1;
}

void
Tree::foreach (nodefunc func, void *arg)
{
	if (left && right)
	{
		(*func)(category, left->category, right->category, arg);
		left->foreach (func, arg);
		right->foreach (func, arg);
	} else {
		(*func)(category, word, -1, arg);
	}
}	

void
Tree::splits (splitfunc func, void *arg)
{
	splits_ (0, func, arg);
}

void
Tree::splits_ (int st, splitfunc func, void *arg)
{
	int n;

	if (left && right)
	{
		n = left->nodes();
		(*func)(st + n, category, left->category, right->category, arg);
		left->splits_ (st, func, arg);
		right->splits_ (st + n, func, arg);
	}
}

void
Tree::traverse (treefunc func, void *arg)
{
	traverse_ (0, func, arg);
}

void
Tree::traverse_ (int st, treefunc func, void *arg)
{
	int n, m;

	if (left && right)
	{
		n = left->nodes ();
		m = right->nodes ();

		(*func)(st, st + n + m - 1,
			category, left->category, right->category, arg);

		left->traverse_ (st, func, arg);
		right->traverse_ (st + n, func, arg);
	}
	
}		
		
void
Tree::print ()
{
	cout << stringify() << endl;
}

string
Tree::stringify (dictionary *dict)
{
	stringstream ss;

	ss << "(" << category << " ";

	if (word < 0)
	{
		ss << (left ? left->stringify(dict) : "-") + " "
		    + (right ? right->stringify(dict) : "-") + ")";
	} else {
		if (dict)
			ss << dict->word(word) << ")";
		else
			ss << word << ")";
	}

	return ss.str();
}

#if 0

void
print_branch (int x, int y, int z, void *arg)
{
	printf("%d->(%d,%d)\n", x, y, z);
}

int
main (int argc, char *argv[])
{
	Tree *tree;

	tree = new Tree (0,
			 new Tree (3, 1),
			 new Tree (1,
				   new Tree (1, 2),
				   new Tree (3,
					     new Tree (2, 3),
					     new Tree (2, 4))));

	tree->print ();

	tree->foreach (print_branch, NULL);

}

#endif

